import boto3
from botocore.exceptions import NoCredentialsError

# Initialize a session using IAM role or environmental credentials
session = boto3.Session(
    region_name='us-west-2'
)

# Create an S3 client
s3_client = session.client('s3')

# Use the Access Point ARN to put or get objects
bucket_arn = 'arn:aws:s3:us-west-2:560961124665:accesspoint/pipeline'

try:
    # Example: Upload a file
    response = s3_client.upload_file(
        'localfile.txt', bucket_arn, 'your-file-key')
except FileNotFoundError:
    print("The file was not found")
except NoCredentialsError:
    print("Credentials not available")



import asyncio
import boto3
import hypersync
import polars as pl
import os
import time

from botocore.exceptions import NoCredentialsError
from hypersync import ColumnMapping, DataType, TransactionField, BlockField

# TODO 4/30/24 - Wait for KANT to give me credentials...
ACCESS_KEY = os.environ(['AWS_ACCESS_KEY'])
SECRET_KEY = os.environ(['AWS_SECRET_KEY'])
REGION = os.environ(['AWS_REGION'])


def upload_file_to_s3(file_name, bucket, object_name=None):
    """
    Upload a file to an S3 bucket

    :param file_name: File to upload
    :param bucket: Bucket to upload to
    :param object_name: S3 object name. If not specified, file_name is used
    :return: True if file was uploaded, else False
    """
    if object_name is None:
        object_name = file_name

    # Create an S3 client
    s3_client = boto3.client('s3', aws_access_key_id=ACCESS_KEY,
                             aws_secret_access_key=SECRET_KEY,
                             region_name=REGION)

    try:
        response = s3_client.upload_file(file_name, bucket, object_name)
        print('file uploaded successfully')
    except FileNotFoundError:
        print("The file was not found")
        return False
    except NoCredentialsError:
        print("Credentials not available")
        return False
    return True


async def collect_blocks_txs():
    """
    Uses the HypersyncClient to collect blocks and transactions from the Ethereum network. This data is stored in a parquet sink.
    """

    client = hypersync.HypersyncClient("https://eth.hypersync.xyz")

    to_block = await client.get_height()
    from_block = to_block - (7200 * 14) 
    batch_size = 10_000  # Define the number of blocks to process per batch

    while from_block < to_block:
        current_to_block = min(from_block + batch_size, to_block)
        print(
            f"Processing blocks {from_block} to {current_to_block}")

        query = client.preset_query_blocks_and_transactions(
            from_block, current_to_block)
        query.max_num_transactions = 1000  # for troubleshooting

        config = hypersync.ParquetConfig(
            path="data",
            hex_output=True,
            batch_size=5000,
            concurrency=10,
            retry=True,
            column_mapping=ColumnMapping(
                transaction={
                    TransactionField.GAS_USED: DataType.FLOAT64,
                    TransactionField.MAX_FEE_PER_BLOB_GAS: DataType.FLOAT64,
                    TransactionField.MAX_PRIORITY_FEE_PER_GAS: DataType.FLOAT64,
                    TransactionField.GAS_PRICE: DataType.FLOAT64,
                    TransactionField.CUMULATIVE_GAS_USED: DataType.FLOAT64,
                    TransactionField.EFFECTIVE_GAS_PRICE: DataType.FLOAT64,
                    TransactionField.NONCE: DataType.INT64,
                    TransactionField.GAS: DataType.FLOAT64,
                },
                block={
                    BlockField.GAS_LIMIT: DataType.FLOAT64,
                    BlockField.GAS_USED: DataType.FLOAT64,
                    BlockField.SIZE: DataType.FLOAT64,
                    BlockField.BLOB_GAS_USED: DataType.FLOAT64,
                    BlockField.EXCESS_BLOB_GAS: DataType.FLOAT64,
                    BlockField.BASE_FEE_PER_GAS: DataType.FLOAT64
                },
            )
        )

        await client.create_parquet_folder(query, config)
        from_block = current_to_block + 1  # Update from_block for the next batch



        # write the parquet file to s3 bucket
        directory = 'data/'
        file_name = 'blocks.parquet'
        bucket_name = 'ethanalytics'

        upload_file_to_s3(directory + file_name, bucket_name)
        # for filename in os.listdir(directory):
        #     if filename.endswith(".parquet"):
        #         file_path = os.path.join(directory, filename)
        #         upload_file_to_s3(file_path, bucket_name)


start_time = time.time()
asyncio.run(collect_blocks_txs())
end_time = time.time()

print(f"Time taken: {end_time - start_time}")



import asyncio
import hypersync
import lancedb
import polars as pl
import time

from hypersync import ColumnMapping, DataType, TransactionField, BlockField
from hypersync_lancedb_pipe.db_manager import Manager


async def head_blocks_txs_sync():
    """
    Uses the HypersyncClient to collect blocks and transactions from the Ethereum network. This data is stored in a parquet sink and written to a local lance database
    """

    client = hypersync.HypersyncClient("https://eth.hypersync.xyz")

    # open the database and get the latest block_number
    db: lancedb.DBConnection = lancedb.connect("blocks")
    table: lancedb.table = db.open_table("blocks")

    # set to_block and from_block to query the desired block range.
    to_block: int = await client.get_height()
    from_block: int = table.to_polars().select('block_number').sort(
        by='block_number', descending=True).collect()['block_number'][0] - 1
    db_batch_size: int = 10_000  # Define the number of blocks to process per batch

    while from_block < to_block:
        current_to_block = min(from_block + db_batch_size, to_block)
        print(
            f"Processing blocks {from_block} to {current_to_block}")

        query = client.preset_query_blocks_and_transactions(
            from_block, current_to_block)

        # Setting this number lower reduces client sync console error messages.
        query.max_num_transactions = 1_000  # for troubleshooting

        config = hypersync.ParquetConfig(
            path="data",
            hex_output=True,
            batch_size=5_000,
            concurrency=10,
            retry=True,
            column_mapping=ColumnMapping(
                transaction={
                    TransactionField.GAS_USED: DataType.FLOAT64,
                    TransactionField.MAX_FEE_PER_BLOB_GAS: DataType.FLOAT64,
                    TransactionField.MAX_PRIORITY_FEE_PER_GAS: DataType.FLOAT64,
                    TransactionField.GAS_PRICE: DataType.FLOAT64,
                    TransactionField.CUMULATIVE_GAS_USED: DataType.FLOAT64,
                    TransactionField.EFFECTIVE_GAS_PRICE: DataType.FLOAT64,
                    TransactionField.NONCE: DataType.INT64,
                    TransactionField.GAS: DataType.FLOAT64,
                },
                block={
                    BlockField.GAS_LIMIT: DataType.FLOAT64,
                    BlockField.GAS_USED: DataType.FLOAT64,
                    BlockField.SIZE: DataType.FLOAT64,
                    BlockField.BLOB_GAS_USED: DataType.FLOAT64,
                    BlockField.EXCESS_BLOB_GAS: DataType.FLOAT64,
                    BlockField.BASE_FEE_PER_GAS: DataType.FLOAT64
                },
            )
        )

        await client.create_parquet_folder(query, config)
        from_block = current_to_block + 1  # Update from_block for the next batch

        # write blocks and transactions data into lancedb tables.
        blocks_table_name = "blocks"
        txs_table_name = "transactions"
        index: str = "block_number"

        # load the dataframe into a polars dataframe and insert into lancedb
        blocks_df = pl.read_parquet(
            f"data/{blocks_table_name}.parquet").rename({'number': 'block_number'})
        txs_df = pl.read_parquet(f"data/{txs_table_name}.parquet")
        # write to lancedb
        lance_manager = Manager()
        lance_manager.write_db(f"{blocks_table_name}", blocks_df,
                               merge_on=index)
        lance_manager.write_db(f"{txs_table_name}",
                               txs_df, merge_on=index)


start_time = time.time()
asyncio.run(head_blocks_txs_sync())
end_time = time.time()

print(f"Time taken: {end_time - start_time}")



import datetime
import lancedb
import polars as pl

from dataclasses import dataclass
from lancedb import DBConnection


@dataclass
class Manager:
    """
    Manager class that contains main functionality to read and write data to a LanceDB Table. 
    """

    def write_db(self, table: lancedb.table, data: pl.DataFrame, merge_on: str = "block_number"):
        """
        Write a dataframe to a LanceDB table. If a dataframe doesn't exist, then create it with the given data. The default
        merge_on value is "block_number" because that is one of the most universal columns for blockchain data.
        """
        # CLEANUP_TIME determines the the frequency that lancedb will clean up the database and compact the files.
        # Increase the number for less frequent cleanups and longer batch update times.
        CLEANUP_TIME = 86400 / 24 / 60
        try:
            db: DBConnection = lancedb.connect(table)
            # Try to open and merge data into existing table.
            table: lancedb.table = db.open_table(table)
            table.compact_files()
            table.to_lance().optimize.optimize_indices()
            table.cleanup_old_versions(
                datetime.timedelta(seconds=CLEANUP_TIME))

            # Perform a "upsert" operation
            table.merge_insert(merge_on)   \
                .when_not_matched_insert_all() \
                .execute(data)

            table.cleanup_old_versions(
                older_than=datetime.timedelta(seconds=CLEANUP_TIME*2), delete_unverified=True
            )
        # if the table doesn't exist, create it with the data
        except FileNotFoundError:
            # Handle the case where the table does not exist
            print(
                f"Creating table {table} because it does not exist.")
            # create table
            table: lancedb.table = db.create_table(name=table, data=data)
